#ifndef __SIMPLE_H_
#define __SIMPLE_H_
#include "resource.h"
#include "MMCSnap.h"
#include "basenode.h"
#include "simpnode.h"
#include "MsgViewNode1.h"
#include "ChooseCompPage.h"
#include "ActiveXNode.h"

class CSimpleData : public CBaseNodeItem<CSimpleData>
{
	CSimpNode* m_pSimpNode;
	CMsgViewNode* m_pMsgNode;
	CActiveXNode* m_pAxNode;
public:

	CSimpleData()
	{
		m_pSimpNode = new CSimpNode;
		m_pMsgNode  = new CMsgViewNode;
		m_pAxNode  = new CActiveXNode;
		// Image indexes may need to be modified depending on the images specific to 
		// the snapin.
		memset(&m_scopeDataItem, 0, sizeof(SCOPEDATAITEM));
		m_scopeDataItem.mask = SDI_STR | SDI_IMAGE | SDI_OPENIMAGE | SDI_PARAM | SDI_CHILDREN;
		m_scopeDataItem.displayname = MMC_CALLBACK;
		m_scopeDataItem.nImage = 0; 		// May need modification
		m_scopeDataItem.nOpenImage = 0; 	// May need modification
		m_scopeDataItem.lParam = (LPARAM) this;
		m_scopeDataItem.cChildren = 2;

		memset(&m_resultDataItem, 0, sizeof(RESULTDATAITEM));
	}

	~CSimpleData()
	{
		delete m_pSimpNode;
		delete m_pMsgNode;
		delete m_pAxNode;
	}
	STDMETHOD(CreatePropertyPages)(LPPROPERTYSHEETCALLBACK lpProvider,long handle, IUnknown* pUnk,
							DATA_OBJECT_TYPES type)
	{
		if (type == CCT_SNAPIN_MANAGER)
		{
			CChooseCompPage* pPage = new CChooseCompPage(handle, true, _T("Simple"));
			HRESULT hr = lpProvider->AddPage(pPage->Create());
			// The second parameter  to the property page class constructor
			// should be true for only one page.
			return S_OK;
		}
		return E_UNEXPECTED;
	}

	STDMETHOD(QueryPagesFor)(DATA_OBJECT_TYPES type)
	{
		if (type == CCT_SNAPIN_MANAGER)
			return S_OK;

		return S_FALSE;
	}

	HRESULT OnShow(IConsole *spConsole, BOOL bShow)
	{
		HRESULT hr = S_FALSE;
		if(bShow)
		{        
			CComQIPtr<IResultData, &IID_IResultData> spResultData(spConsole);
			CComQIPtr<IHeaderCtrl, &IID_IHeaderCtrl> spHeaderCtrl(spConsole);
			HRESULT hr = spResultData->DeleteAllRsltItems();
			hr = spHeaderCtrl->InsertColumn( 0, L"Column - 1", 0,  MMCLV_AUTO);
			hr = spHeaderCtrl->InsertColumn( 1, L"Column - 2", 0, MMCLV_AUTO);
		}	 
		return hr;
	}
	HRESULT OnExpand(BOOL bExpand, IConsole *pConsole, HSCOPEITEM parent);
};

class CSimple;

class CSimpleComponent : public CComObjectRootEx<CComSingleThreadModel>,
	public CSnapInObjectRoot<2, CSimple >,
	public IExtendPropertySheetImpl<CSimpleComponent>,
	public IExtendContextMenuImpl<CSimpleComponent>,
	public IExtendControlbarImpl<CSimpleComponent>,
	public IComponentImpl<CSimpleComponent>
{
public:
BEGIN_COM_MAP(CSimpleComponent)
	COM_INTERFACE_ENTRY(IComponent)
    COM_INTERFACE_ENTRY(IExtendPropertySheet)
    COM_INTERFACE_ENTRY(IExtendContextMenu)
    COM_INTERFACE_ENTRY(IExtendControlbar)
END_COM_MAP()

public:
	CSimpleComponent():m_ipDisplayHelp(NULL)
	{
	}

	STDMETHOD(Initialize)(LPCONSOLE lpConsole)
	{
		HRESULT hr=IComponentImpl<CSimpleComponent>::Initialize(lpConsole);
		_ASSERT( S_OK == hr );
		hr = m_spConsole->QueryInterface(IID_IDisplayHelp, (void **)&m_ipDisplayHelp);
		_ASSERT( S_OK == hr );
		return hr;
	}
	
	STDMETHOD(Destroy)(long cookie)
	{
		if (m_ipDisplayHelp) 
		{
			m_ipDisplayHelp->Release();
			m_ipDisplayHelp = NULL;
		}
		HRESULT hr=IComponentImpl<CSimpleComponent>::Destroy(cookie);
		return hr;
	}

	STDMETHOD(Notify)(LPDATAOBJECT lpDataObject, MMC_NOTIFY_TYPE event, long arg, long param)
	{
		if (lpDataObject != NULL)
			return IComponentImpl<CSimpleComponent>::Notify(lpDataObject, event, arg, param);

		return E_NOTIMPL;
	}

	IDisplayHelp* m_ipDisplayHelp;
};

class CSimple : public CComObjectRootEx<CComSingleThreadModel>,
public CSnapInObjectRoot<1, CSimple>,
	public IComponentDataImpl<CSimple, CSimpleComponent>,
	public IExtendPropertySheetImpl<CSimple>,
	public IExtendContextMenuImpl<CSimple>,
	public IPersistStreamInit,
	public CComCoClass<CSimple, &CLSID_Simple>,
	ISnapinHelp2
{
public:
	CSimple()
	{
		m_pNode = new CSimpleData;
		_ASSERTE(m_pNode != NULL);
		m_pComponentData = this;
	}

	~CSimple()
	{
		delete m_pNode;
		m_pNode = NULL;
	}

BEGIN_COM_MAP(CSimple)
	COM_INTERFACE_ENTRY(IComponentData)
    COM_INTERFACE_ENTRY(IExtendPropertySheet)
    COM_INTERFACE_ENTRY(IExtendContextMenu)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISnapinHelp2)
	COM_INTERFACE_ENTRY(ISnapinHelp)
END_COM_MAP()

DECLARE_REGISTRY_RESOURCEID(IDR_SIMPLE)

DECLARE_NOT_AGGREGATABLE(CSimple)

/////ISnapinHelp2
	 HRESULT STDMETHODCALLTYPE GetLinkedTopics(/* [out] */ LPOLESTR __RPC_FAR *lpCompiledHelpFiles)
	 {
		 return S_FALSE;
	 }
/////ISnapinHelp
     HRESULT STDMETHODCALLTYPE GetHelpTopic(/* [out] */ LPOLESTR __RPC_FAR *lpCompiledHelpFile);

	STDMETHOD(GetClassID)(CLSID *pClassID)
	{
		*pClassID = CLSID_Simple;
		return S_OK;
	}	

	STDMETHOD(IsDirty)()
	{
		return S_FALSE; //implement the dirty mechanism if needed
	}

	STDMETHOD(Load)(IStream *pStm)
	{
		char buff[MAX_PATH];
		return pStm->Read(buff, sizeof(buff), NULL); //change to something usefull
	}

	STDMETHOD(Save)(IStream *pStm, BOOL fClearDirty)
	{
		char buff[MAX_PATH];
		sprintf(buff,"local");
		return pStm->Write(buff, sizeof(buff), NULL);//change to something usefull
	}

	STDMETHOD(GetSizeMax)(ULARGE_INTEGER *pcbSize)
	{
		pcbSize->QuadPart = MAX_PATH; //change to something usefull
		return S_OK;
	}

	STDMETHOD(InitNew)()
	{
		return S_OK;
	}

	STDMETHOD(Initialize)(LPUNKNOWN pUnknown);

	static void WINAPI ObjectMain(bool bStarting)
	{
		if (bStarting)
			CSnapInItem::Init();
	}
	static HRESULT LoadImages(IImageList*);

	STDMETHOD(Notify)(LPDATAOBJECT lpDataObject,MMC_NOTIFY_TYPE event,long arg,long param)
	{
		HRESULT hr = S_FALSE;
		if(lpDataObject != NULL)
		{
			hr= IComponentDataImpl<CSimple, CSimpleComponent>::Notify(lpDataObject,event,arg,param);
		}
		else
		{
			if(event == MMCN_PROPERTY_CHANGE)
			{
				if(arg) //scope
				{

				}
				else //result
				{

				}
			}
		}
		return hr;
	}
};

class ATL_NO_VTABLE CSimpleAbout : public ISnapinAbout,
	public CComObjectRoot,
	public CComCoClass< CSimpleAbout, &CLSID_SimpleAbout>
{
	HBITMAP				m_hSmallImage;
    HBITMAP				m_hLargeImage;
    HBITMAP				m_hSmallImageOpen;
	HICON m_hAppIcon;
public:
	CSimpleAbout()
	{
		m_hSmallImage = (HBITMAP)LoadImage(_Module.GetModuleInstance(), MAKEINTRESOURCE(IDB_SIMPLE_16), IMAGE_BITMAP, 16, 16, LR_LOADTRANSPARENT);
		m_hLargeImage = (HBITMAP)LoadImage(_Module.GetModuleInstance(), MAKEINTRESOURCE(IDB_SIMPLE_32), IMAGE_BITMAP, 32, 32, LR_LOADTRANSPARENT);
		m_hSmallImageOpen = (HBITMAP)LoadImage(_Module.GetModuleInstance(), MAKEINTRESOURCE(IDB_SMOPEN), IMAGE_BITMAP, 16, 16, LR_LOADTRANSPARENT);
		m_hAppIcon = LoadIcon(_Module.GetModuleInstance(), MAKEINTRESOURCE(IDI_APPICON));
	}
	~CSimpleAbout()
	{
		::DeleteObject(m_hSmallImage);
		::DeleteObject(m_hLargeImage);
		::DeleteObject(m_hSmallImageOpen);
		::DestroyIcon(m_hAppIcon);
	}

	DECLARE_REGISTRY(CSimpleAbout, _T("SimpleAbout.1"), _T("SimpleAbout.1"), IDS_SIMPLE_DESC, THREADFLAGS_BOTH);

	BEGIN_COM_MAP(CSimpleAbout)
		COM_INTERFACE_ENTRY(ISnapinAbout)
	END_COM_MAP()

	STDMETHOD(GetSnapinDescription)(LPOLESTR *lpDescription)
	{
		USES_CONVERSION;
		TCHAR szBuf[256];
		if (::LoadString(_Module.GetResourceInstance(), IDS_SIMPLE_DESC, szBuf, 256) == 0)
			return E_FAIL;

		*lpDescription = (LPOLESTR)CoTaskMemAlloc((lstrlen(szBuf) + 1) * sizeof(OLECHAR));
		if (*lpDescription == NULL)
			return E_OUTOFMEMORY;

		ocscpy(*lpDescription, T2OLE(szBuf));

		return S_OK;
	}

	STDMETHOD(GetProvider)(LPOLESTR *lpName)
	{
		USES_CONVERSION;
		TCHAR szBuf[256];
		if (::LoadString(_Module.GetResourceInstance(), IDS_SIMPLE_PROVIDER, szBuf, 256) == 0)
			return E_FAIL;

		*lpName = (LPOLESTR)CoTaskMemAlloc((lstrlen(szBuf) + 1) * sizeof(OLECHAR));
		if (*lpName == NULL)
			return E_OUTOFMEMORY;

		ocscpy(*lpName, T2OLE(szBuf));

		return S_OK;
	}

	STDMETHOD(GetSnapinVersion)(LPOLESTR *lpVersion)
	{
		USES_CONVERSION;
		TCHAR szBuf[256];
		if (::LoadString(_Module.GetResourceInstance(), IDS_SIMPLE_VERSION, szBuf, 256) == 0)
			return E_FAIL;

		*lpVersion = (LPOLESTR)CoTaskMemAlloc((lstrlen(szBuf) + 1) * sizeof(OLECHAR));
		if (*lpVersion == NULL)
			return E_OUTOFMEMORY;

		ocscpy(*lpVersion, T2OLE(szBuf));

		return S_OK;
	}

	STDMETHOD(GetSnapinImage)(HICON *hAppIcon)
	{
		*hAppIcon = m_hAppIcon;
		return S_OK;
	}

	STDMETHOD(GetStaticFolderImage)(HBITMAP *hSmallImage,
		HBITMAP *hSmallImageOpen,
		HBITMAP *hLargeImage,
		COLORREF *cMask)
	{
		*hSmallImage = m_hSmallImage;
		*hSmallImageOpen = m_hSmallImageOpen;
		*hLargeImage = m_hLargeImage;
		*cMask = RGB(0, 128, 128);
		if (*hSmallImage == NULL || *hLargeImage == NULL || 
         *hSmallImageOpen == NULL)
			return E_FAIL;
		else
	        return S_OK;
	}
};

#endif
